//
//  ImageConverter.m
//  BlackAndWhite
//
//  Created by David Phillip Oster on 10/18/09.
//  Copyright 2009 David Phillip Oster. All rights reserved.
//

#import "ImageConverter.h"

enum {
  kRed = 0,
  kGreen = 1,
  kBlue = 2
};

@interface ImageConverter()

- (BOOL)startUp:(UIImage *)src;

- (UIImage *)closeDown;

- (void)cleanup;

@end

@implementation ImageConverter

- (void)dealloc {
  [self cleanup];
  [super dealloc];
}

- (void)cleanup {
  if (destContext_ != NULL) {
    CGContextRelease(destContext_);
    destContext_ = NULL;
  }
  if (data_ != NULL) {
    free(data_);
    data_ = NULL;
  }
}


- (UIImage *)toBlackAndWhite:(UIImage *)src red:(float)r green:(float)g blue:(float)b {
  NSAssert(fabs(r+g+b - 1.0) < 1.0e-6, @"Coefficients should sum to 1.");
  if ([self startUp:src]) {
    unsigned char *p = (unsigned char *)data_; // point to front of scan line.
    unsigned char *pEnd = p +(height_ * rowBytes_);
    for (; p < pEnd; p += 4) {
      unsigned char gray = r*p[kRed] + g*p[kGreen] + b*p[kBlue];
      p[kRed] = p[kGreen] = p[kBlue] = gray;
    }
  }
  return [self closeDown];
}

- (BOOL)startUp:(UIImage *)src {
  BOOL result = NO;
  CGSize s = [src size];
  height_ = s.height;
  width_ = s.width;
  rowBytes_ = width_ * 4;
  data_ = (unsigned char *)malloc(height_ * rowBytes_);
  if (data_ != NULL) {
    // See http://developer.apple.com/mac/library/qa/qa2001/qa1037.html .
    destContext_ = CGBitmapContextCreate(
        data_, width_, height_, 8, rowBytes_,
        CGColorSpaceCreateDeviceRGB(), kCGImageAlphaPremultipliedLast);
    if (destContext_ != NULL) {
      CGRect srcFrame = CGRectMake(0, 0, width_, height_); 
      CGContextDrawImage(destContext_, srcFrame, [src CGImage]);
      result = YES;
    }
  }
  return result;
}

- (UIImage *)closeDown {
  UIImage *dest = nil;
  if (destContext_ != NULL) {
    CGImageRef destCGImage = CGBitmapContextCreateImage(destContext_);
    dest = [[[UIImage alloc] initWithCGImage:destCGImage] autorelease];
    CGImageRelease(destCGImage);
  }
  [self cleanup];
  return dest;
}

@end
